# Labkey as a docker container

This git project contains everything required to start labkey as a docker container.

## Installation

### Prepare

- Install [`docker`][docker] and [`docker-compose`][dockerCompose]
- create a user `labkey` with `docker` privileges

```bash
sudo adduser labkey
sudo usermod -G docker labkey
```

- clone the software 

```bash
sudo su labkey
cd
git clone https://git0.fmf.uni-lj.si/studen/labkeyDocker.git
```

### Database

Create directory or copy appropriate database cluster to `labkeyDocker/postgresql/data`:

```bash
mkdir -p labkeyDocker/postgresql/data
```

Links are OK, too. Exact path is set in `bin/env.sh`, see below.


Check if database runs:

```bash
labkeyDocker/bin/startPGonly.sh
docker logs config-labkey-db-1
labkeyDocker/bin/stopLabkey.sh
```

### Files and external modules for LabKey

The files and externalModules locations are set in `bin/env.sh`, see `bin/env.sh.sample`:
```bash
export EXTERNAL_MODULES=$HOME/externalModules
export LABKEY_FILES=/data/labkey/files
```

### Install LabKey binaries

- copy binaries from `https://labkey-public.fmf.uni-lj.si/labkey/Resources to `$HOME/packages`. Remember the version number for next step.
- run `labkeyDocker/bin/installLabkey.sh 22` to unzip the LabKey binary code and make appropriate links.
- copy `labkeyDocker/bin/env.sh.sample` to `labkeyDocker/bin/env.sh` and set a secret `$PGPASS`, other settings are probably OK, 
  check `$DOCKERCOMPOSE`
- copy 
- set the same value as `password` in `labkeyDocker/tomcat/conf/Catalina/localhost/labkey.xml` 

### Permissions

To run as non-root user labkey, files should have the proper permissions. I believe that starting from scratch, 
the neccesary files will be created with appropriate permissions. If using an old implementation or in case of troubles, the following directories should be owned by labkey:labkey on host machine:

```bash
~/labkey/externalModules
~/labkey/files
```

### Start containers
  
Run `bin/startLabkey.sh`. This will do several things:
   - Download PostgreSQL image and initialize a persistent database at `~/postgresl/data`. 
  - Build a combined tomcat/R image from a recent version of tomcat
  - Copy and link files from home to the server
  - Start the server

### Check the installation
  
In principle, the service should be running at `http://localhost:8443/labkey`. Check output of inidividual 
  components by:
  
```bash
  docker logs config-db-1
  docker logs config-web-1
  docker exec -it config-db-1 bash
  
```

### Add ggplot2 to R

Connect to running instance of labkey:

```bash
docker exec -it config_labkey-web_1 bash
```

Start R:

```bash
R
```

In R, install`ggplot2` with dependencies:

```R
install.packages('ggplot2', repos='http://cran.us.r-project.org', dependencies=TRUE)
```

#### Install ggpubr

For ggpubr, install `cmake` first. When in config_labkey-web_1 do:
```bash
apt update
apt install cmake
```

Then, in R:
```R
install.packages('ggpubr')
```

### Manage postgresql

Create `${POSTGRES_DIR}` directory as user that will be running the container.
If you have trouble with initial setup, the base can be reset by deleting everything under `postgresql/data`.


### Expected errors
  
Errors of the type `500: Unexpected server error` are common and related 

# Pre-building images

In the latest version, images can be pre-built using docker hub as a repository for such images lacking adaptation to individual working environment.

To create such images, do
```bash
$ docker build -f config/web/Dockerfile21 -t andrejstuden/nix:x21 config/web
$ docker login -u andrejstuden docker.io
$ docker push andrejstuden/nix:x21
```
  
[docker]: https://docs.docker.com/engine/install/
[dockerCompose]: https://docs.docker.com/compose/install/
